STRICT = true
DEBUG = true

require 'zoetrope'
--inspect = require 'inspect'
--_ = require 'underscore'

Player = Animation:extend {
   image = 'data/player.png',
   height = 32,
   width = 32,
   sequences = {
      stand = { frames = { 1 }, fps = 1 },
      walk = { frames = { 2, 3 }, fps = 5 },
      jump = { frames = { 4 }, fps = 1 }
   },
   onNew = function (self)
              self.velocity.y = 0
              self.maxVelocity.y = 400
           end,
   onStartFrame = function (self)
                     -- this is all in startframe so it happens before
                     -- physics calc at beginning of update

                     -- jumping/falling updates could go in EndFrame...
                     self.falling = self.velocity.y > 0
                     if self.falling then self.jumping = false end
                     --print(self.jumping, self.falling)

                     if not self.onGround then
                        self:play('jump')
                     end

                     self.velocity.x = 0
                     self.acceleration.y = 800

                     if the.keys:pressed('left') then
                        self.velocity.x = -200
                        if self.onGround then self:play('walk') end
                     elseif the.keys:pressed('right') then
                        self.velocity.x = 200
                        if self.onGround then self:play('walk') end
                     else
                        if self.onGround then self:play('stand') end
                     end

                     if the.keys:justPressed('up') and self.onGround then
                        self.velocity.y = -400
                        self.jumping = true
                     end
                  end,
   onUpdate = function (self)
                 -- needs to happen right before collision
                 self.onGround = false
                end,
   onCollide = function (self, other, xOverlap, yOverlap)
                  -- seriously, why does this even fire?
                  if other == the.view.map then return end

                  --print(string.format('col s{x=%i y=%i w=%i h=%i} %s', self.x, self.y, self.width, self.height, tostring(other)))
                  --print('vel.x:'..self.velocity.x.." vel.y:"..self.velocity.y)

                  -- assumption: any other collision is with a solid map tile
                  if yOverlap > xOverlap then
                     other:displace(self)

                     if self.velocity.x > 0 then
                        self.onWall = 'right'
                     elseif self.velocity.x < 0 then
                        self.onWall = 'left'
                     else
                        print '??'
                     end
                  elseif xOverlap > yOverlap then
                     -- check if we've moved since collisions were generated
                     local xov, yov = self:overlap(other.x, other.y,
                                                   other.width, other.height)
                     if xov ~= 0 and yov ~= 0 then
                        --print('y collision')
                        if self.velocity.y > 0 then
                           self.onGround = true
                        end

                        self.velocity.y = 0
                        other:displace(self)
                        self.jumping = false
                     end
                  else
                     print('??')
                  end

               end,
}

GameView = View:extend {
   onNew = function (self)
              self:loadLayers('data/map.lua')
              self.focus = the.player
              self:clampTo(self.map)
           end,
   onUpdate = function (self)
                 --print('tick')
                 the.player:collide(self.map)
                 --self.map:collide(the.player)
              end
}

the.app = App:new {
   onRun = function (self)
              self.view = GameView:new()
              --print(inspect(_(the.app):keys()))
              self.console:watch('onGround', 'the.player.onGround')
              self.console:watch('onWall', 'the.player.onWall')
           end,
   onUpdate = function (self, dt)
                 if the.keys:justPressed('escape') and 
                   not self.console.visible then
                    love.event.quit()
                 end
              end
}